@@ -8,15 +8,19 @@ use Sub::Exporter -setup => {
     groups  => { default => [ qw(resource router missing GET POST PUT DELETE HEAD OPTIONS TRACE CONNECT) ] }
 };
 
-our $VERSION = '0.11';
+our $VERSION = '0.20';
 
-sub new { bless { routes => [] } };
+sub new {
+    my $class = shift;
+    bless { @_, routes => [] };
+}
 
 our (%METHS, $ROUTER);
 
-sub router(&) {
-    local $ROUTER = __PACKAGE__->new;
-    shift->();
+sub router(&;@) {
+    my ($block, @settings) = @_;
+    local $ROUTER = __PACKAGE__->new(@settings);
+    $block->();
     return $ROUTER;
 }
 
@@ -28,6 +32,12 @@ sub resource ($&) {
     # Let HEAD use GET if not specified.
     $METHS{HEAD} ||= $METHS{GET};
 
+    # Add OPTIONS if requested.
+    if ($ROUTER->{auto_options} && !$METHS{OPTIONS}) {
+        my $methods = join(', ' => 'OPTIONS', keys %METHS);
+        $METHS{OPTIONS} = sub { [200, ['Allow', $methods], []] };
+    }
+
     # Add the route.
     push @{ $ROUTER->{routes} }, Router::Simple::Route->new(
         $path, { meths => { %METHS } }
@@ -175,6 +185,21 @@ something like this:
 But of course you can abstract that into a controller or other code that the
 HTTP method simply dispatches to.
 
+If you wish the router to create an C<OPTIONS> handler for you, pass the
+C<auto_options> parameter to C<router>:
+
+    $router = router {
+        resource '/blog/{year}/{month}' => sub {
+            GET  { [200, [], [ $template->render({ posts => \@posts }) ] };
+            POST { push @posts, new_post(shift); [200, [], ['ok']] };
+        };
+    } auto_options => 1;
+
+With C<auto_options> enabled, Router::Resource will look at the methods
+defined for a resource to define the C<OPTIONS> handler. In this example,
+C<$router>'s C<OPTIONS> method will specify that C<GET>, C<HEAD>, and
+C<OPTIONS> are valid for C</blog/{year}/{month}>.
+
 =head2 Dispatching
 
 Use the C<dispatch> method to have the router dispatch HTTP requests. For a
@@ -349,7 +374,7 @@ including:
 
 =head1 Copyright and License
 
-Copyright (c) 2010 David E. Wheeler. Some Rights Reserved.
+Copyright (c) 2010-2011 David E. Wheeler. Some Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
